const _ = require('lodash');

function calculateLibrariesChangeset (currentLibraries, newLibraries) {
  const changeset = [],
    currentLibrariesNames = Object.keys(currentLibraries),
    newLibrariesNames = Object.keys(newLibraries),

    addedLibraries = _.difference(newLibrariesNames, currentLibrariesNames),
    deletedLibraries = _.difference(currentLibrariesNames, newLibrariesNames),
    commonLibraries = _.intersection(newLibrariesNames, currentLibrariesNames);

  for (const library of addedLibraries) {
    changeset.push({
      element: `library/${newLibraries[library].slug}`,
      operation: 'add',
      value: newLibraries[library]
    });
  }

  for (const library of deletedLibraries) {
    changeset.push({
      element: `library/${currentLibraries[library].slug}`,
      operation: 'remove'
    });
  }

  for (const library of commonLibraries) {
    const currentLibrary = currentLibraries[library],
      newLibrary = newLibraries[library];

    if (!_.isEqual(currentLibrary, newLibrary)) {
      changeset.push({
        element: `library/${currentLibrary.slug}`,
        operation: 'update',
        value: newLibrary
      });
    }
  }

  return changeset;
}

function calculateCategoriesChangeset (currentCategories, newCategories) {
  const changeset = [],
    currentCategoriesNames = Object.keys(currentCategories),
    newCategoriesNames = Object.keys(newCategories),

    addedCategories = _.difference(newCategoriesNames, currentCategoriesNames),
    deletedCategories = _.difference(currentCategoriesNames, newCategoriesNames),
    commonCategories = _.intersection(newCategoriesNames, currentCategoriesNames);

  for (const category of addedCategories) {
    changeset.push({
      element: `category/${newCategories[category].slug}`,
      operation: 'add',
      value: newCategories[category]
    });
  }

  for (const category of deletedCategories) {
    changeset.push({
      element: `category/${currentCategories[category].slug}`,
      operation: 'remove'
    });
  }

  for (const category of commonCategories) {
    const currentCategory = currentCategories[category],
      newCategory = newCategories[category];

    if (!_.isEqual(currentCategory, newCategory)) {
      changeset.push({
        element: `category/${currentCategory.slug}`,
        operation: 'update',
        value: newCategory
      });
    }
  }

  return changeset;
}

function calculateRulesChangeset (currentRules, newRules) {
  const changeset = [],
    currentRulesNames = Object.keys(currentRules),
    newRulesNames = Object.keys(newRules),

    addedRules = _.difference(newRulesNames, currentRulesNames),
    deletedRules = _.difference(currentRulesNames, newRulesNames),
    commonRules = _.intersection(newRulesNames, currentRulesNames);

  for (const rule of addedRules) {
    changeset.push({
      element: `rule/${newRules[rule].slug}`,
      operation: 'add',
      value: newRules[rule]
    });
  }

  for (const rule of deletedRules) {
    changeset.push({
      element: `rule/${currentRules[rule].slug}`,
      operation: 'remove'
    });
  }

  for (const rule of commonRules) {
    const currentRule = currentRules[rule],
      newRule = newRules[rule];

    if (!_.isEqual(currentRule, newRule)) {
      changeset.push({
        element: `rule/${currentRule.slug}`,
        operation: 'update',
        value: newRule
      });
    }
  }

  return changeset;
}

// Should we check if the metafile is valid like each library/category/rule has its property defined??
// If a category is deleted then all the rules in that category should be deleted as well?? Do we want to add these to the changeset??
function calculateChangeset (currentMetafile, newMetafile) {
  const changeset = [],
    currentLibraries = currentMetafile.libraries,
    newLibraries = newMetafile.libraries,
    currentCategories = currentMetafile.categories,
    newCategories = newMetafile.categories,
    currentRules = currentMetafile.rules,
    newRules = newMetafile.rules,

    librariesChangeset = calculateLibrariesChangeset(currentLibraries, newLibraries),
    categoriesChangeset = calculateCategoriesChangeset(currentCategories, newCategories),
    rulesChangeset = calculateRulesChangeset(currentRules, newRules);

  changeset.push(...librariesChangeset, ...categoriesChangeset, ...rulesChangeset);

  return changeset;
}

module.exports = calculateChangeset;

