const changeset = require('@postman/rules-config-changeset');

const currentMetafile = {
    libraries: {
      'postman-openapi-library': {
        name: 'Postman OpenAPI Library',
        slug: 'postman-openapi-library',
        logo: '/_common-assets/api-governance/images/postman_logo.svg',
        illustrationLink: 'https://res.cloudinary.com/postman/image/upload/v1627371094/curated_list/v1/Explore-Application_Security.png',
        description: {
          short: 'Rules recommended by Postman to kickstart an API Governance program.',
          long: 'Rules curated by Postman staff to promote building consistent APIs for API producers and consumers.'
        }
      }
    },
    categories: {
      'category-1': {
        slug: 'category-1',
        name: 'Category 1',
        description: 'OAS spectral rule recommended by stoplight.'
      },
      'category-2': {
        slug: 'category-2',
        name: 'Category 2',
        description: ''
      }
    },
    vulnerabilities: {
      'vulnerability-1': {
        slug: 'vulnerability-1',
        name: 'Vulnerability 1',
        category: 'category-1',
        config: {
          tags: [
            'governance'
          ]
        }
      },
      'vulnerability-2': {
        slug: 'vulnerability-2',
        name: 'Vulnerability 2',
        category: 'category-2',
        config: {
          tags: [
            'governance'
          ]
        }
      }
    },
    rules: {
      'rule-1': {
        name: 'rule 1',
        slug: 'rule-1',
        vulnerability: 'references.vulnerability-1',
        config: {
          spectralRule: {
            json: {
              OPENAPI_SCHEMA_RULE_0005: {
                then: {
                  field: 'security',
                  function: 'defined'
                },
                given: '$',
                formats: [
                  'oas3'
                ],
                message: 'Security field is not defined'
              }
            }
          }
        },
        engine: 'SPECTRAL 6.0',
        entity: {
          type: 'SCHEMA',
          config: {
            type: [
              'OpenAPI 3.0',
              'OpenAPI 3.1'
            ]
          }
        },
        severity: 'WARN',
        message: 'The contact object should be defined and non empty in the info object',
        mitigation: {
          description: '',
          possibleFixUrl: ''
        }
      }
    }
  },

  newMetafile = {
    libraries: {
      'postman-openapi-library-1': {
        name: 'Postman OpenAPI Library',
        slug: 'postman-openapi-library-1',
        logo: '/_common-assets/api-governance/images/postman_logo.svg',
        illustrationLink: 'https://res.cloudinary.com/postman/image/upload/v1627371094/curated_list/v1/Explore-Application_Security.png',
        description: {
          short: 'Rules recommended by Postman to kickstart an API Governance program.',
          long: 'Rules curated by Postman staff to promote building consistent APIs for API producers and consumers.'
        }
      }
    },
    categories: {
      'category-1': {
        slug: 'category-1',
        name: 'Category 1',
        description: 'OAS spectral rule recommended by stoplight.'
      },
      'category-2': {
        slug: 'category-2',
        name: 'Category 2',
        description: ''
      }
    },
    vulnerabilities: {
      'vulnerability-1': {
        slug: 'vulnerability-1',
        name: 'Vulnerability 1',
        category: 'category-1',
        config: {
          tags: [
            'governance'
          ]
        }
      },
      'vulnerability-2': {
        slug: 'vulnerability-2',
        name: 'Vulnerability 2',
        category: 'category-2',
        config: {
          tags: [
            'governance'
          ]
        }
      }
    },
    rules: {
      'rule-1': {
        name: 'rule 1',
        slug: 'rule-1',
        vulnerability: 'references.vulnerability-1',
        config: {
          spectralRule: {
            json: {
              OPENAPI_SCHEMA_RULE_0005: {
                then: {
                  field: 'security',
                  function: 'defined'
                },
                given: '$',
                formats: [
                  'oas3'
                ],
                message: 'Security field is not defined'
              }
            }
          }
        },
        engine: 'SPECTRAL 6.0',
        entity: {
          type: 'SCHEMA',
          config: {
            type: [
              'OpenAPI 3.0',
              'OpenAPI 3.1'
            ]
          }
        },
        severity: 'WARN',
        message: 'The contact object should be defined and non empty in the info object',
        mitigation: {
          description: '',
          possibleFixUrl: ''
        }
      }
    }
  };


console.log(changeset(currentMetafile, newMetafile));